@@ -64,20 +64,38 @@ def tile_grid_for_epsg(epsg, bbox=None, tile_size=(256, 256), res=None):
         return TileGrid(epsg, is_geodetic=True, bbox=bbox, tile_size=tile_size, res=res)
     return TileGrid(epsg, bbox=bbox, tile_size=tile_size, res=res)
 
-default_bboxs = {
-    SRS(900913): (-20037508.342789244,
-                  -20037508.342789244,
-                   20037508.342789244,
-                   20037508.342789244),
-    SRS(3857): (-20037508.342789244,
-                -20037508.342789244,
-                 20037508.342789244,
-                 20037508.342789244),
-    SRS(4326): (-180, -90, 180, 90),
-}
+
+# defer loading of default bbox since custom proj settings
+# are not loaded on import time
+class _default_bboxs(object):
+    _defaults = {
+        900913: (-20037508.342789244,
+                 -20037508.342789244,
+                  20037508.342789244,
+                  20037508.342789244),
+        3857: (-20037508.342789244,
+               -20037508.342789244,
+                20037508.342789244,
+                20037508.342789244),
+        4326: (-180, -90, 180, 90),
+    }
+    defaults = None
+    def get(self, key, default=None):
+        try:
+            return self[key]
+        except KeyError:
+            return default
+    def __getitem__(self, key):
+        if self.defaults is None:
+            defaults = {}
+            for epsg, bbox in self._defaults.iteritems():
+                defaults[SRS(epsg)] = bbox
+            self.defaults = defaults
+        return self.defaults[key]
+default_bboxs = _default_bboxs()
 
 def tile_grid(srs=None, bbox=None, bbox_srs=None, tile_size=(256, 256),
-              res=None, res_factor=2.0,
+              res=None, res_factor=2.0, threshold_res=None,
               num_levels=None, min_res=None, max_res=None,
               stretch_factor=None, max_shrink_factor=None,
               align_with=None
@@ -110,7 +128,7 @@ def tile_grid(srs=None, bbox=None, bbox_srs=None, tile_size=(256, 256),
     else:
         res = resolutions(min_res, max_res, res_factor, num_levels, bbox, tile_size)
     
-    return TileGrid(srs, bbox=bbox, tile_size=tile_size, res=res,
+    return TileGrid(srs, bbox=bbox, tile_size=tile_size, res=res, threshold_res=threshold_res,
                     stretch_factor=stretch_factor, max_shrink_factor=max_shrink_factor)
 
 def aligned_resolutions(min_res=None, max_res=None, res_factor=2.0, num_levels=None,
@@ -221,7 +239,7 @@ class TileGrid(object):
     spheroid_a = 6378137.0 # for 900913
     
     def __init__(self, srs=900913, bbox=None, tile_size=(256, 256), res=None,
-                 is_geodetic=False, levels=None,
+                 threshold_res=None, is_geodetic=False, levels=None,
                  stretch_factor=None, max_shrink_factor=None):
         """
         :param stretch_factor: allow images to be scaled up by this factor
@@ -268,6 +286,7 @@ class TileGrid(object):
 
         self.levels = len(res)
         self.resolutions = res
+        self.threshold_res = threshold_res
         
         self.grid_sizes = self._calc_grids()
     
@@ -329,9 +348,28 @@ class TileGrid(object):
                                              l1_res-50, l1_res*0.91, l1_res*0.89, 8000.0)]
         [0, 0, 1, 1, 1, 1, 2, 5]
         """
+        prev_l_res = self.resolutions[0]
+        threshold = None
+        thresholds = []
+        if self.threshold_res:
+            thresholds = self.threshold_res[::-1]
+            threshold = thresholds.pop()
+        
+        threshold_result = None
         for level, l_res in enumerate(self.resolutions):
+            if threshold and prev_l_res > threshold >= l_res:
+                if res > threshold:
+                    return level-1
+                elif res >= l_res:
+                    return level
+                threshold = thresholds.pop() if thresholds else None
+            
+            if threshold_result is not None:
+                return threshold_result
+
             if l_res <= res*self.stretch_factor:
-                return level
+                threshold_result = level
+            prev_l_res = l_res
         return level
     
     def tile(self, x, y, level):
@@ -889,3 +927,88 @@ def bbox_contains(one, two):
         ): return True
     
     return False
+
+def deg_to_m(deg):
+    return deg * (6378137 * 2 * math.pi) / 360
+
+OGC_PIXLE_SIZE = 0.00028 #m/px
+
+def ogc_scale_to_res(scale):
+    return scale * OGC_PIXLE_SIZE
+def res_to_ogc_scale(res):
+    return res / OGC_PIXLE_SIZE
+
+def resolution_range(min_res=None, max_res=None, max_scale=None, min_scale=None):
+    if min_scale == max_scale == min_res == max_res == None:
+        return ResolutionRange(None, None)
+    if min_res or max_res:
+        if not max_scale and not min_scale:
+            return ResolutionRange(min_res, max_res)
+    elif max_scale or min_scale:
+        if not min_res and not max_res:
+            min_res = ogc_scale_to_res(max_scale)
+            max_res = ogc_scale_to_res(min_scale)
+            return ResolutionRange(min_res, max_res)
+            
+    raise ValueError('requires either min_res/max_res or max_scale/min_scale')
+
+class ResolutionRange(object):
+    def __init__(self, min_res, max_res):
+        self.min_res = min_res
+        self.max_res = max_res
+        
+        if min_res and max_res:
+            assert min_res > max_res
+    
+    def scale_denominator(self):
+        min_scale = res_to_ogc_scale(self.max_res) if self.max_res else None
+        max_scale = res_to_ogc_scale(self.min_res) if self.min_res else None
+        return min_scale, max_scale
+    
+    def scale_hint(self):
+        """
+        Returns the min and max diagonal resolution.
+        """
+        min_res = self.min_res
+        max_res = self.max_res
+        if min_res:
+            min_res = math.sqrt(2*min_res**2)
+        if max_res:
+            max_res = math.sqrt(2*max_res**2)
+        return min_res, max_res
+    
+    def contains(self, bbox, size, srs):
+        width, height = bbox_size(bbox)
+        if srs.is_latlong:
+            width = deg_to_m(width)
+            height = deg_to_m(height)
+        
+        x_res = width/size[0]
+        y_res = height/size[1]
+        
+        if self.min_res:
+            min_res = self.min_res + 1e-6
+            if min_res <= x_res or min_res <= y_res:
+                return False
+        if self.max_res:
+            max_res = self.max_res - 1e-6
+            if max_res > x_res or max_res > y_res:
+                return False
+        
+        return True
+    
+    def __repr__(self):
+        return '<ResolutionRange(min_res=%.3f, max_res=%.3f)>' % (
+            self.min_res, self.max_res)
+    
+
+def max_with_none(a, b):
+    if a is None or b is None:
+        return None
+    else:
+        return max(a, b)
+
+def merge_resolution_range(a, b):
+    if a and b:
+        return ResolutionRange(max_with_none(a.min_res, b.min_res), min(a.max_res, b.max_res))
+    return None
