@@ -16,29 +16,35 @@
 
 import platform
 
+__all__ = ['Image', 'ImageColor', 'ImageDraw', 'ImageFont', 'ImagePalette', 'quantize']
+
 if platform.system() == "Java":
     from jil import Image, ImageColor, ImageDraw, ImageFont
+    Image, ImageColor, ImageDraw, ImageFont # prevent pyflakes warnings
     
     class ImagePalette(object):
         def __init__(self, *args, **kw):
             raise NotImplementedError()
             
-    def quantize(img, colors=256, alpha=False, defaults=None):
+    def quantize_jil(img, colors=256, alpha=False, defaults=None):
         return img.convert('P', palette=Image.ADAPTIVE, colors=colors)
-    
+    quantize = quantize_jil
 else:
     try:
         from PIL import Image, ImageColor, ImageDraw, ImageFont, ImagePalette
+        Image, ImageColor, ImageDraw, ImageFont, ImagePalette # prevent pyflakes warnings
     except ImportError:
         import Image, ImageColor, ImageDraw, ImageFont, ImagePalette
+        Image, ImageColor, ImageDraw, ImageFont, ImagePalette # prevent pyflakes warnings
     
-    def quantize(img, colors=256, alpha=False, defaults=None):
+    def quantize_pil(img, colors=256, alpha=False, defaults=None):
         if hasattr(Image, 'FASTOCTREE'):
             if not alpha:
                 img = img.convert('RGB')
             img = img.quantize(colors, Image.FASTOCTREE)
         else:
             if alpha:
+                img.load() # split might fail if image is not loaded
                 alpha = img.split()[3]
                 img = img.convert('RGB').convert('P', palette=Image.ADAPTIVE, colors=colors-1)
                 mask = Image.eval(alpha, lambda a: 255 if a <=128 else 0)
@@ -48,4 +54,5 @@ else:
             else:
                 img = img.convert('RGB').convert('P', palette=Image.ADAPTIVE, colors=colors)
            
-        return img
\ No newline at end of file
+        return img
+    quantize = quantize_pil
\ No newline at end of file
