@@ -88,12 +88,12 @@ class WMSMapRequestParams(RequestParams):
     del _set_srs
 
     def _get_transparent(self):
-         """
-         ``True`` if transparent is set to true, otherwise ``False``.
-         """
-         if self.get('transparent', 'false').lower() == 'true':
-             return True
-         return False
+        """
+        ``True`` if transparent is set to true, otherwise ``False``.
+        """
+        if self.get('transparent', 'false').lower() == 'true':
+            return True
+        return False
     def _set_transparent(self, transparent):
         self.params['transparent'] = str(transparent).lower()
     transparent = property(_get_transparent, _set_transparent)
@@ -183,6 +183,13 @@ class WMSMapRequest(WMSRequest):
                             non_strict=non_strict, **kw)
     
     def validate(self):
+        self.validate_param()
+        self.validate_bbox()
+        self.validate_format()
+        self.validate_srs()
+        self.validate_styles()
+    
+    def validate_param(self):
         missing_param = []
         for param in self.expected_param:
             if self.non_strict and param in self.non_strict_params:
@@ -195,11 +202,6 @@ class WMSMapRequest(WMSRequest):
                 self.params['format'] = 'image/png'
             raise RequestError('missing parameters ' + str(missing_param),
                                request=self)
-        
-        self.validate_bbox()
-        self.validate_format()
-        self.validate_srs()
-        self.validate_styles()
     
     def validate_bbox(self):
         x0, y0, x1, y1 = self.params.bbox
@@ -221,7 +223,7 @@ class WMSMapRequest(WMSRequest):
     def validate_styles(self):
         if 'styles' in self.params:
             styles = self.params['styles']
-            if styles.replace(',', '').strip() != '':
+            if not set(styles.split(',')).issubset(set(['default', ''])):
                 raise RequestError('unsupported styles: ' + self.params['styles'],
                                    code='StyleNotDefined', request=self)
         
@@ -256,6 +258,13 @@ class WMS100MapRequest(WMSMapRequest):
         del params['service']
         return params
 
+class WMS110MapRequest(WMSMapRequest):
+    fixed_params = {'request': 'GetMap', 'version': '1.1.0', 'service': 'WMS'}
+    xml_exception_handler = exception.WMS110ExceptionHandler
+    
+    def adapt_to_111(self):
+        del self.params['wmtver']
+
 class WMS111MapRequest(WMSMapRequest):
     fixed_params = {'request': 'GetMap', 'version': '1.1.1', 'service': 'WMS'}
     xml_exception_handler = exception.WMS111ExceptionHandler
@@ -312,6 +321,42 @@ class WMS130MapRequest(WMSMapRequest):
         new_req.params.switch_bbox()
         return new_req
 
+class WMSLegendGraphicRequestParams(WMSMapRequestParams):
+    """
+    RequestParams for WMS GetLegendGraphic requests.
+    """
+    def _set_layer(self, value):
+        self.params['layer'] = value
+    
+    def _get_layer(self):
+        """
+        Layer for which to produce legend graphic.
+        """
+        return self.params.get('layer')
+    layer = property(_get_layer, _set_layer)
+    del _set_layer
+    del _get_layer
+        
+    @property
+    def sld_version(self):
+        """
+        Specification version for SLD-specification
+        """
+        return self.params.get('sld_version')
+    
+    
+    def _set_scale(self, value):
+        self.params['scale'] = value
+    
+    def _get_scale(self):
+        if self.params.get('scale') is not None:
+            return float(self['scale'])
+        return None
+    
+    scale = property(_get_scale,_set_scale)
+    del _set_scale
+    del _get_scale
+
 class WMSFeatureInfoRequestParams(WMSMapRequestParams):
     """
     RequestParams for WMS GetFeatureInfo requests.
@@ -343,6 +388,34 @@ class WMSFeatureInfoRequestParams(WMSMapRequestParams):
 class WMS130FeatureInfoRequestParams(WMSFeatureInfoRequestParams):
     switch_bbox = _switch_bbox
 
+class WMSLegendGraphicRequest(WMSMapRequest):    
+    request_params = WMSLegendGraphicRequestParams
+    request_handler_name = 'legendgraphic'
+    non_strict_params = set(['sld_version', 'scale'])
+    fixed_params = {'request': 'GetLegendGraphic', 'service': 'WMS', 'sld_version': '1.1.0'}
+    expected_param = ['version', 'request', 'layer', 'format', 'sld_version']
+    
+    def validate(self):
+        self.validate_param()
+        self.validate_format()
+        self.validate_sld_version()
+
+    def validate_sld_version(self):
+        if self.params.get('sld_version', '1.1.0') != '1.1.0':
+            raise RequestError('invalid sld_version ' + self.params.get('sld_version'),
+                                request=self)
+    
+
+class WMS111LegendGraphicRequest(WMSLegendGraphicRequest):
+    fixed_params = WMSLegendGraphicRequest.fixed_params.copy()
+    fixed_params['version'] = '1.1.1'
+    xml_exception_handler = exception.WMS111ExceptionHandler
+    
+class WMS130LegendGraphicRequest(WMSLegendGraphicRequest):
+    fixed_params = WMSLegendGraphicRequest.fixed_params.copy()
+    fixed_params['version'] = '1.3.0'
+    xml_exception_handler = exception.WMS130ExceptionHandler
+
 class WMSFeatureInfoRequest(WMSMapRequest):
     non_strict_params = set(['format', 'styles'])
     
@@ -357,7 +430,15 @@ class WMS111FeatureInfoRequest(WMSFeatureInfoRequest):
     fixed_params = WMS111MapRequest.fixed_params.copy()
     fixed_params['request'] = 'GetFeatureInfo'
     expected_param = WMSMapRequest.expected_param[:] + ['query_layers', 'x', 'y']
-    
+
+class WMS110FeatureInfoRequest(WMSFeatureInfoRequest):
+    request_params = WMSFeatureInfoRequestParams
+    xml_exception_handler = exception.WMS110ExceptionHandler
+    request_handler_name = 'featureinfo'
+    fixed_params = WMS110MapRequest.fixed_params.copy()
+    fixed_params['request'] = 'GetFeatureInfo'
+    expected_param = WMSMapRequest.expected_param[:] + ['query_layers', 'x', 'y']
+
 class WMS100FeatureInfoRequest(WMSFeatureInfoRequest):
     request_params = WMSFeatureInfoRequestParams
     xml_exception_handler = exception.WMS100ExceptionHandler
@@ -430,6 +511,15 @@ class WMS100CapabilitiesRequest(WMSCapabilitiesRequest):
         return exception.WMS100ExceptionHandler()
     
 
+class WMS110CapabilitiesRequest(WMSCapabilitiesRequest):
+    capabilities_template = 'wms110capabilities.xml'
+    mime_type = 'application/vnd.ogc.wms_xml'
+    fixed_params = {'request': 'GetCapabilities', 'version': '1.1.0', 'service': 'WMS'}
+    
+    @property
+    def exception_handler(self):
+        return exception.WMS110ExceptionHandler()
+
 class WMS111CapabilitiesRequest(WMSCapabilitiesRequest):
     capabilities_template = 'wms111capabilities.xml'
     mime_type = 'application/vnd.ogc.wms_xml'
@@ -470,12 +560,17 @@ class Version(object):
 request_mapping = {Version('1.0.0'): {'featureinfo': WMS100FeatureInfoRequest,
                                       'map': WMS100MapRequest,
                                       'capabilities': WMS100CapabilitiesRequest},
+                   Version('1.1.0'): {'featureinfo': WMS110FeatureInfoRequest,
+                                       'map': WMS110MapRequest,
+                                       'capabilities': WMS110CapabilitiesRequest},
                    Version('1.1.1'): {'featureinfo': WMS111FeatureInfoRequest,
                                       'map': WMS111MapRequest,
-                                      'capabilities': WMS111CapabilitiesRequest},
+                                      'capabilities': WMS111CapabilitiesRequest,
+                                      'legendgraphic': WMS111LegendGraphicRequest},
                    Version('1.3.0'): {'featureinfo': WMS130FeatureInfoRequest,
                                       'map': WMS130MapRequest,
-                                      'capabilities': WMS130CapabilitiesRequest},
+                                      'capabilities': WMS130CapabilitiesRequest,
+                                      'legendgraphic': WMS130LegendGraphicRequest},
                    }
 
 
@@ -497,6 +592,8 @@ def _parse_request_type(req):
             return 'featureinfo'
         elif request_type in ('getcapabilities', 'capabilities'):
             return 'capabilities'
+        elif request_type in ('getlegendgraphic',):
+            return 'legendgraphic'
         else:
             return request_type
     else:
@@ -512,7 +609,7 @@ def negotiate_version(version):
     >>> negotiate_version(Version('1.1.1'))
     Version('1.1.1')
     >>> negotiate_version(Version('1.1.0'))
-    Version('1.0.0')
+    Version('1.1.0')
     """
     supported_versions = request_mapping.keys()
     supported_versions.sort()
