@@ -24,7 +24,7 @@ import math
 import threading
 from itertools import izip
 from mapproxy.proj import Proj, transform, set_datapath, ProjInitError
-from mapproxy.config import base_config, abspath
+from mapproxy.config import base_config
 
 import logging
 log = logging.getLogger(__name__)
@@ -65,8 +65,8 @@ class TransformationError(Exception):
 _proj_initalized = False
 def _init_proj():
     global _proj_initalized
-    if not _proj_initalized and base_config().srs.get('proj_data_dir'):
-        proj_data_dir = abspath(base_config().srs.proj_data_dir)
+    if not _proj_initalized and 'proj_data_dir' in base_config().srs:
+        proj_data_dir = base_config().srs['proj_data_dir']
         log.info('loading proj data from %s', proj_data_dir)
         set_datapath(proj_data_dir)
         _proj_initalized = True
@@ -74,6 +74,9 @@ def _init_proj():
 _thread_local = threading.local()
 def SRS(srs_code):
     _init_proj()
+    if isinstance(srs_code, _SRS):
+        return srs_code
+    
     srs_code = _clean_srs_code(srs_code)
     
     if not hasattr(_thread_local, 'srs_cache'):
@@ -308,6 +311,9 @@ class _SRS(object):
         "SRS('EPSG:4326')"
         """
         return "SRS('%s')" % (self.srs_code,)
+    
+    def __hash__(self):
+        return hash(self.srs_code)
 
 
 def generate_envelope_points(bbox, n):
