@@ -22,68 +22,39 @@ from mapproxy.platform.image import Image
 import functools
 
 from cStringIO import StringIO
-from webtest import TestApp
-import mapproxy.config
 from mapproxy.srs import SRS
-from mapproxy.wsgiapp import make_wsgi_app 
 from mapproxy.request.wms import WMS100MapRequest, WMS111MapRequest, WMS130MapRequest, \
                                  WMS111FeatureInfoRequest, WMS111CapabilitiesRequest, \
                                  WMS130CapabilitiesRequest, WMS100CapabilitiesRequest, \
                                  WMS100FeatureInfoRequest, WMS130FeatureInfoRequest, \
+                                 WMS110MapRequest, WMS110FeatureInfoRequest, \
+                                 WMS110CapabilitiesRequest, \
                                  wms_request
-from mapproxy.test.unit.test_grid import assert_almost_equal_bbox
 from mapproxy.test.image import is_jpeg, is_png, tmp_image
 from mapproxy.test.http import mock_httpd
 from mapproxy.test.helper import validate_with_dtd, validate_with_xsd
 from nose.tools import eq_, assert_almost_equal
 
-global_app = None
+from mapproxy.test.system import module_setup, module_teardown, SystemTest, make_base_config
+
+test_config = {}
+base_config = make_base_config(test_config)
 
 def setup_module():
-    fixture_dir = os.path.join(os.path.dirname(__file__), 'fixture')
-    fixture_layer_conf = os.path.join(fixture_dir, 'layer.yaml')
-    fixture_cache_data = os.path.join(fixture_dir, 'cache_data')
-    mapproxy.config.base_config().debug_mode = True
-    mapproxy.config.base_config().services_conf = fixture_layer_conf
-    mapproxy.config.base_config().cache.base_dir = fixture_cache_data
-    mapproxy.config.base_config().image.paletted = False
-    mapproxy.config._service_config = None
-    
-    global global_app
-    global_app = TestApp(make_wsgi_app(fixture_layer_conf), use_unicode=False)
+    module_setup(test_config, 'layer.yaml', with_cache_data=True)
 
 def teardown_module():
-    mapproxy.config._config = None
-    mapproxy.config._service_config = None
+    module_teardown(test_config)
 
 def test_invalid_url():
-    global_app.get('/invalid?fop', status=404)
-
-class WMSTest(object):
-    def setup(self):
-        self.app = global_app
-        self.created_tiles = []
-    
-    def created_tiles_filenames(self):
-        base_dir = mapproxy.config.base_config().cache.base_dir
-        for filename in self.created_tiles:
-            yield os.path.join(base_dir, filename)
-    
-    def _test_created_tiles(self):
-        for filename in self.created_tiles_filenames():
-            if not os.path.exists(filename):
-                assert False, "didn't found tile " + filename
-    
-    def teardown(self):
-        self._test_created_tiles()
-        for filename in self.created_tiles_filenames():
-            if os.path.exists(filename):
-                os.remove(filename)
-
+    test_config['app'].get('/invalid?fop', status=404)
 
 def is_100_capa(xml):
     return validate_with_dtd(xml, dtd_name='wms/1.0.0/capabilities_1_0_0.dtd')
 
+def is_110_capa(xml):
+    return validate_with_dtd(xml, dtd_name='wms/1.1.0/capabilities_1_1_0.dtd')
+
 def is_111_exception(xml, msg=None, code=None, re_msg=None):
     eq_(xml.xpath('/ServiceExceptionReport/@version')[0], '1.1.1')
     if msg:
@@ -101,14 +72,16 @@ def is_111_capa(xml):
 def is_130_capa(xml):
     return validate_with_xsd(xml, xsd_name='wms/1.3.0/capabilities_1_3_0.xsd')
 
-class TestWMSVersionDispatch(object):
-    def setup(self):
-        self.app = global_app
-        
+
+class WMSTest(SystemTest):
+    config = test_config
+
+class TestCoverageWMS(WMSTest):
+    
     def test_unknown_version_110(self):
         resp = self.app.get('http://localhost/service?SERVICE=WMS&REQUEST=GetCapabilities'
                             '&VERSION=1.1.0')
-        assert is_100_capa(resp.lxml)
+        assert is_110_capa(resp.lxml)
     def test_unknown_version_113(self):
         resp = self.app.get('http://localhost/service?SERVICE=WMS&REQUEST=GetCapabilities'
                             '&VERSION=1.1.3')
@@ -148,11 +121,11 @@ class TestWMS111(WMSTest):
         xml = resp.lxml
         eq_(xml.xpath('//GetMap//OnlineResource/@xlink:href',
                       namespaces=dict(xlink="http://www.w3.org/1999/xlink"))[0],
-            'http://localhost:80/service?')
+            'http://localhost/service?')
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
                               'wms_merge', 'tms_cache', 'wms_cache_multi',
-                              'wms_cache_link_single'])
+                              'wms_cache_link_single', 'wms_cache_110'])
         eq_(layer_names, expected_names)
         assert validate_with_dtd(xml, dtd_name='wms/1.1.1/WMS_MS_Capabilities.dtd')
     
@@ -186,11 +159,27 @@ class TestWMS111(WMSTest):
     def test_invalid_srs(self):
         self.common_map_req.params['srs'] = 'EPSG:1234'
         resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'application/vnd.ogc.se_xml')
         is_111_exception(resp.lxml, 'unsupported srs: EPSG:1234', 'InvalidSRS')
+
+    def test_get_map_unknown_style(self):
+        self.common_map_req.params['styles'] = 'unknown'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'application/vnd.ogc.se_xml')
+        is_111_exception(resp.lxml, 'unsupported styles: unknown', 'StyleNotDefined')
+    
+    def test_get_map_default_style(self):
+        self.common_map_req.params['styles'] = 'default'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/png')
+        print resp.body
+        data = StringIO(resp.body)
+        assert is_png(data)
+        assert Image.open(data).mode == 'RGB'
     
     def test_get_map_png(self):
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/png'
+        eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
@@ -198,7 +187,7 @@ class TestWMS111(WMSTest):
     def test_get_map_png_transparent(self):
         self.common_map_req.params['transparent'] = 'True'
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/png'
+        eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGBA'
@@ -206,7 +195,7 @@ class TestWMS111(WMSTest):
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/jpeg'
+        eq_(resp.content_type, 'image/jpeg')
         assert is_jpeg(StringIO(resp.body))
     
     def test_get_map_xml_exception(self):
@@ -262,7 +251,6 @@ class TestWMS111(WMSTest):
                 eq_(resp.content_type, 'image/png')
         
     def test_get_map_use_direct_with_transform(self):
-        bbox_900913 = [1110868.98971,6444038.14317,1229263.18538,6623564.86585]
         with tmp_image((200, 200), format='png') as img:
             expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
                                       '&REQUEST=GetMap&HEIGHT=303&SRS=EPSG%3A900913&styles='
@@ -417,9 +405,9 @@ class TestWMS111(WMSTest):
             eq_(resp.body, 'info')
     
     def test_get_featureinfo_missing_params_strict(self):
-        request_parser = self.app.app.handlers['service'].request_parser
+        request_parser = self.app.app.application.handlers['service'].request_parser
         try:
-            self.app.app.handlers['service'].request_parser = \
+            self.app.app.application.handlers['service'].request_parser = \
                 functools.partial(wms_request, strict=True)
         
             del self.common_fi_req.params['format']
@@ -429,7 +417,7 @@ class TestWMS111(WMSTest):
             assert 'missing parameters' in xml.xpath('//ServiceException/text()')[0]
             assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
         finally:
-            self.app.app.handlers['service'].request_parser = request_parser
+            self.app.app.application.handlers['service'].request_parser = request_parser
     
     def test_get_featureinfo_not_queryable(self):
         self.common_fi_req.params['query_layers'] = 'tms_cache'
@@ -441,6 +429,171 @@ class TestWMS111(WMSTest):
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
         assert 'tms_cache is not queryable' in xml.xpath('//ServiceException/text()')[0]
         assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
+    
+
+class TestWMS110(WMSTest):
+    def setup(self):
+        WMSTest.setup(self)
+        self.common_req = WMS110MapRequest(url='/service?', param=dict(service='WMS', 
+             version='1.1.0'))
+        self.common_map_req = WMS110MapRequest(url='/service?', param=dict(service='WMS', 
+             version='1.1.0', bbox='-180,0,0,80', width='200', height='200',
+             layers='wms_cache', srs='EPSG:4326', format='image/png',
+             styles='', request='GetMap'))
+        self.common_fi_req = WMS110FeatureInfoRequest(url='/service?',
+            param=dict(x='10', y='20', width='200', height='200', layers='wms_cache',
+                       format='image/png', query_layers='wms_cache_110', styles='',
+                       bbox='1000,400,2000,1400', srs='EPSG:900913'))
+                       
+    def test_wms_capabilities(self):
+        req = WMS110CapabilitiesRequest(url='/service?').copy_with_request_params(self.common_req)
+        resp = self.app.get(req)
+        eq_(resp.content_type, 'application/vnd.ogc.wms_xml')
+        xml = resp.lxml
+        eq_(xml.xpath('//GetMap//OnlineResource/@xlink:href',
+                      namespaces=dict(xlink="http://www.w3.org/1999/xlink"))[0],
+            'http://localhost/service?')
+        
+        llbox = xml.xpath('//Capability/Layer/LatLonBoundingBox')[0]
+        # some clients don't like 90deg north/south
+        assert_almost_equal(float(llbox.attrib['miny']), -89.999999, 6)
+        assert_almost_equal(float(llbox.attrib['maxy']), 89.999999, 6)
+        assert_almost_equal(float(llbox.attrib['minx']), -180.0, 6)
+        assert_almost_equal(float(llbox.attrib['maxx']), 180.0, 6)
+        
+        layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
+        expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
+                              'wms_merge', 'tms_cache', 'wms_cache_multi',
+                              'wms_cache_link_single', 'wms_cache_110'])
+        eq_(layer_names, expected_names)
+        assert validate_with_dtd(xml, dtd_name='wms/1.1.0/capabilities_1_1_0.dtd')
+
+    def test_invalid_layer(self):
+        self.common_map_req.params['layers'] = 'invalid'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'application/vnd.ogc.se_xml')
+        xml = resp.lxml
+        eq_(xml.xpath('/ServiceExceptionReport/@version')[0], '1.1.0')
+        eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code')[0], 'LayerNotDefined')
+        eq_(xml.xpath('//ServiceException/text()')[0], 'unknown layer: invalid')
+        assert validate_with_dtd(xml, dtd_name='wms/1.1.0/exception_1_1_0.dtd')
+
+    def test_invalid_format(self):
+        self.common_map_req.params['format'] = 'image/ascii'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'application/vnd.ogc.se_xml')
+        xml = resp.lxml
+        eq_(xml.xpath('/ServiceExceptionReport/@version')[0], '1.1.0')
+        eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code')[0], 'InvalidFormat')
+        eq_(xml.xpath('//ServiceException/text()')[0], 'unsupported image format: image/ascii')
+        assert validate_with_dtd(xml, dtd_name='wms/1.1.0/exception_1_1_0.dtd')
+    
+    def test_invalid_format_img_exception(self):
+        self.common_map_req.params['format'] = 'image/ascii'
+        self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/png')
+        assert is_png(StringIO(resp.body))
+    
+    def test_missing_format_img_exception(self):
+        del self.common_map_req.params['format']
+        self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/png')
+        assert is_png(StringIO(resp.body))
+
+    def test_invalid_srs(self):
+        self.common_map_req.params['srs'] = 'EPSG:1234'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'application/vnd.ogc.se_xml')
+        xml = resp.lxml
+        eq_(xml.xpath('/ServiceExceptionReport/@version')[0], '1.1.0')
+        eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code')[0], 'InvalidSRS')
+        eq_(xml.xpath('//ServiceException/text()')[0], 'unsupported srs: EPSG:1234')
+        assert validate_with_dtd(xml, dtd_name='wms/1.1.0/exception_1_1_0.dtd')
+        
+    def test_get_map_png(self):
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/png')
+        data = StringIO(resp.body)
+        assert is_png(data)
+        assert Image.open(data).mode == 'RGB'
+    
+    def test_get_map_png_transparent(self):
+        self.common_map_req.params['transparent'] = 'True'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/png')
+        data = StringIO(resp.body)
+        assert is_png(data)
+        assert Image.open(data).mode == 'RGBA'
+    
+    def test_get_map_jpeg(self):
+        self.common_map_req.params['format'] = 'image/jpeg'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'image/jpeg')
+        assert is_jpeg(StringIO(resp.body))
+    
+    def test_get_map_xml_exception(self):
+        self.common_map_req.params['bbox'] = '0,0,90,90'
+        resp = self.app.get(self.common_map_req)
+        eq_(resp.content_type, 'application/vnd.ogc.se_xml')
+        xml = resp.lxml
+        eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
+        assert 'No response from URL' in xml.xpath('//ServiceException/text()')[0]
+        assert validate_with_dtd(xml, 'wms/1.1.0/exception_1_1_0.dtd')
+
+    def test_get_map(self):
+        self.created_tiles.append('wms_cache_EPSG900913/01/000/000/001/000/000/001.jpeg')
+        with tmp_image((256, 256), format='jpeg') as img:
+            expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
+                                      '&REQUEST=GetMap&HEIGHT=256&SRS=EPSG%3A900913&styles='
+                                      '&VERSION=1.1.1&BBOX=0.0,0.0,20037508.3428,20037508.3428'
+                                      '&WIDTH=256'},
+                            {'body': img.read(), 'headers': {'content-type': 'image/jpeg'}})
+            with mock_httpd(('localhost', 42423), [expected_req]):
+                self.common_map_req.params['bbox'] = '0,0,180,90'
+                resp = self.app.get(self.common_map_req)
+                assert 35000 < int(resp.headers['Content-length']) < 75000
+                eq_(resp.content_type, 'image/png')
+    
+    def test_get_map_110(self):
+        self.created_tiles.append('wms_cache_110_EPSG900913/01/000/000/001/000/000/001.jpeg')
+        with tmp_image((256, 256), format='jpeg') as img:
+            expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fjpeg'
+                                      '&REQUEST=GetMap&HEIGHT=256&SRS=EPSG%3A900913&styles='
+                                      '&VERSION=1.1.0&BBOX=0.0,0.0,20037508.3428,20037508.3428'
+                                      '&WIDTH=256'},
+                            {'body': img.read(), 'headers': {'content-type': 'image/jpeg'}})
+            with mock_httpd(('localhost', 42423), [expected_req]):
+                self.common_map_req.params['bbox'] = '0,0,180,90'
+                self.common_map_req.params['layers'] = 'wms_cache_110'
+                resp = self.app.get(self.common_map_req)
+                assert 35000 < int(resp.headers['Content-length']) < 75000
+                eq_(resp.content_type, 'image/png')
+    
+    def test_get_featureinfo(self):
+        expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
+                                  '&REQUEST=GetFeatureInfo&HEIGHT=200&SRS=EPSG%3A900913'
+                                  '&VERSION=1.1.0&BBOX=1000.0,400.0,2000.0,1400.0&styles='
+                                  '&WIDTH=200&QUERY_LAYERS=foo,bar&X=10&Y=20'},
+                        {'body': 'info', 'headers': {'content-type': 'text/plain'}})
+        with mock_httpd(('localhost', 42423), [expected_req]):
+            resp = self.app.get(self.common_fi_req)
+            print resp.body
+            eq_(resp.content_type, 'text/plain')
+            eq_(resp.body, 'info')
+            
+    def test_get_featureinfo_not_queryable(self):
+        self.common_fi_req.params['query_layers'] = 'tms_cache'
+        self.common_fi_req.params['exceptions'] = 'application/vnd.ogc.se_xml'
+        resp = self.app.get(self.common_fi_req)
+        print resp.body
+        eq_(resp.content_type, 'application/vnd.ogc.se_xml')
+        xml = resp.lxml
+        eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
+        assert 'tms_cache is not queryable' in xml.xpath('//ServiceException/text()')[0]
+        assert validate_with_dtd(xml, 'wms/1.1.0/exception_1_1_0.dtd')
+    
 
 class TestWMS100(WMSTest):
     def setup(self):
@@ -465,7 +618,7 @@ class TestWMS100(WMSTest):
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
                               'wms_merge', 'tms_cache', 'wms_cache_multi',
-                              'wms_cache_link_single'])
+                              'wms_cache_link_single', 'wms_cache_110'])
         eq_(layer_names, expected_names)
         #TODO srs
         assert validate_with_dtd(xml, dtd_name='wms/1.0.0/capabilities_1_0_0.dtd')
@@ -511,27 +664,27 @@ class TestWMS100(WMSTest):
     
     def test_get_map_png(self):
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/png'
+        eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         eq_(Image.open(data).mode, 'RGB')
         
     def test_get_map_png_transparent_paletted(self):
         try:
-            mapproxy.config.base_config().image.paletted = True
+            base_config().image.paletted = True
             self.common_map_req.params['transparent'] = 'True'
             resp = self.app.get(self.common_map_req)
-            resp.content_type = 'image/png'
+            eq_(resp.content_type, 'image/png')
             data = StringIO(resp.body)
             assert is_png(data)
             assert Image.open(data).mode == 'P'
         finally:
-            mapproxy.config.base_config().image.paletted = False
+            base_config().image.paletted = False
             
     def test_get_map_png_transparent(self):
         self.common_map_req.params['transparent'] = 'True'
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/png'
+        eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGBA'
@@ -539,7 +692,7 @@ class TestWMS100(WMSTest):
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/jpeg'
+        eq_(resp.content_type, 'image/jpeg')
         assert is_jpeg(StringIO(resp.body))
     
     def test_get_map_xml_exception(self):
@@ -582,7 +735,9 @@ class TestWMS100(WMSTest):
         assert 'tms_cache is not queryable' in xml.xpath('//WMTException/text()')[0]
 
 ns130 = {'wms': 'http://www.opengis.net/wms',
-         'ogc': 'http://www.opengis.net/ogc'}
+         'ogc': 'http://www.opengis.net/ogc',
+         'sld': 'http://www.opengis.net/sld',
+         'xlink': 'http://www.w3.org/1999/xlink'}
 
 def eq_xpath(xml, xpath, expected, namespaces=None):
     eq_(xml.xpath(xpath, namespaces=namespaces)[0], expected)
@@ -602,7 +757,7 @@ class TestWMS130(WMSTest):
             param=dict(i='10', j='20', width='200', height='200', layers='wms_cache_130',
                        format='image/png', query_layers='wms_cache_130', styles='',
                        bbox='1000,400,2000,1400', crs='EPSG:900913'))
-        
+
     def test_wms_capabilities(self):
         req = WMS130CapabilitiesRequest(url='/service?').copy_with_request_params(self.common_req)
         resp = self.app.get(req)
@@ -614,7 +769,7 @@ class TestWMS130(WMSTest):
                                     namespaces=ns130))
         expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
                               'wms_merge', 'tms_cache', 'wms_cache_multi',
-                              'wms_cache_link_single'])
+                              'wms_cache_link_single', 'wms_cache_110'])
         eq_(layer_names, expected_names)
         assert is_130_capa(xml)
     
@@ -668,7 +823,7 @@ class TestWMS130(WMSTest):
     
     def test_get_map_png(self):
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/png'
+        eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
@@ -676,7 +831,7 @@ class TestWMS130(WMSTest):
     def test_get_map_png_transparent(self):
         self.common_map_req.params['transparent'] = 'True'
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/png'
+        eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGBA'
@@ -684,7 +839,7 @@ class TestWMS130(WMSTest):
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
-        resp.content_type = 'image/jpeg'
+        eq_(resp.content_type, 'image/jpeg')
         assert is_jpeg(StringIO(resp.body))
     
     def test_get_map_xml_exception(self):
@@ -733,6 +888,7 @@ class TestWMS130(WMSTest):
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
+    
 
 if sys.platform != 'win32':
     class TestWMSLinkSingleColorImages(WMSTest):
@@ -759,7 +915,7 @@ if sys.platform != 'win32':
                     resp = self.app.get(self.common_map_req)
                     eq_(resp.content_type, 'image/jpeg')
             
-                base_dir = mapproxy.config.base_config().cache.base_dir
+                base_dir = base_config().cache.base_dir
                 single_loc = os.path.join(base_dir, real_name)
                 tile_loc = os.path.join(base_dir, link_name)
                 assert os.path.exists(single_loc)
