@@ -15,6 +15,10 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import with_statement
+
+import time
+import sys
+
 from mapproxy.client.http import HTTPClient, HTTPClientError
 from mapproxy.client.tile import TMSClient, TileClient, TileURLTemplate
 from mapproxy.client.wms import WMSClient, WMSInfoClient
@@ -23,9 +27,8 @@ from mapproxy.request.wms import wms_request, WMS111MapRequest, WMS100MapRequest
                                  WMS130MapRequest, WMS111FeatureInfoRequest
 from mapproxy.srs import bbox_equals, SRS
 from mapproxy.request import Request, url_decode
-from mapproxy.config import base_config
 from mapproxy.test.http import mock_httpd, query_eq, assert_query_eq, make_wsgi_env
-from mapproxy.test.helper import assert_re, TempFiles
+from mapproxy.test.helper import assert_re, TempFile
 
 from nose.tools import eq_
 from nose.plugins.skip import SkipTest
@@ -36,6 +39,12 @@ TESTSERVER_URL = 'http://%s:%s' % TESTSERVER_ADDRESS
 class TestHTTPClient(object):
     def setup(self):
         self.client = HTTPClient()
+    
+    def test_post(self):
+        with mock_httpd(TESTSERVER_ADDRESS, [({'path': '/service?foo=bar', 'method': 'POST'},
+                                              {'status': '200', 'body': ''})]):
+            self.client.open(TESTSERVER_URL + '/service', data="foo=bar")
+    
     def test_internal_error_response(self):
         try:
             with mock_httpd(TESTSERVER_ADDRESS, [({'path': '/'},
@@ -73,14 +82,6 @@ class TestHTTPClient(object):
             assert_re(e.args[0], r'No response .* \(http://localhost.*\): Connection refused')
         else:
             assert False, 'expected HTTPClientError'
-    def test_internal_error(self):
-        try:
-            self.client.open('http://localhost:53871', invalid_key='argument')
-        except HTTPClientError, e:
-            assert_re(e.args[0], r'Internal HTTP error \(http://localhost.*\): TypeError')
-        else:
-            assert False, 'expected HTTPClientError'
-
     
     def test_https_no_ssl_module_error(self):
         from mapproxy.client import http
@@ -101,12 +102,10 @@ class TestHTTPClient(object):
         old_ssl = http.ssl
         try:
             http.ssl = None
-            base_config().http.ssl_no_cert_checks = True
-            self.client = HTTPClient('https://trac.osgeo.org/')
+            self.client = HTTPClient('https://trac.osgeo.org/', insecure=True)
             self.client.open('https://trac.osgeo.org/')
         finally:
             http.ssl = old_ssl
-            base_config().http.ssl_no_cert_checks = False
     
     def test_https_valid_cert(self):
         try:
@@ -114,11 +113,10 @@ class TestHTTPClient(object):
         except ImportError:
             raise SkipTest()
         
-        with TempFiles(1) as tmp:
-            with open(tmp[0], 'w') as f:
+        with TempFile() as tmp:
+            with open(tmp, 'w') as f:
                 f.write(OSGEO_CERT)
-            base_config().http.ssl_ca_certs = tmp[0]
-            self.client = HTTPClient('https://trac.osgeo.org/')
+            self.client = HTTPClient('https://trac.osgeo.org/', ssl_ca_certs=tmp)
             self.client.open('https://trac.osgeo.org/')
     
     def test_https_invalid_cert(self):
@@ -127,14 +125,56 @@ class TestHTTPClient(object):
         except ImportError:
             raise SkipTest()
         
-        with TempFiles(1) as tmp:
-            base_config().http.ssl_ca_certs = tmp[0]
-            self.client = HTTPClient('https://trac.osgeo.org/')
+        with TempFile() as tmp:
+            self.client = HTTPClient('https://trac.osgeo.org/', ssl_ca_certs=tmp)
             try:
                 self.client.open('https://trac.osgeo.org/')
             except HTTPClientError, e:
                 assert_re(e.args[0], r'Could not verify connection to URL')
         
+    def test_timeouts(self):
+        test_req = ({'path': '/', 'req_assert_function': lambda x: time.sleep(0.5) or True},
+                    {'body': 'nothing'})
+
+        import mapproxy.client.http
+
+        old_timeout = mapproxy.client.http._max_set_timeout
+        mapproxy.client.http._max_set_timeout = None
+
+        client1 = HTTPClient(timeout=0.1)
+        client2 = HTTPClient(timeout=0.2)
+        with mock_httpd(TESTSERVER_ADDRESS, [test_req]):
+            try:
+                start = time.time()
+                resp = client1.open(TESTSERVER_URL+'/')
+            except HTTPClientError, ex:
+                assert 'timed out' in ex.args[0]
+            else:
+                assert False, 'HTTPClientError expected'
+            duration1 = time.time() - start
+
+        with mock_httpd(TESTSERVER_ADDRESS, [test_req]):
+            try:
+                start = time.time()
+                resp = client2.open(TESTSERVER_URL+'/')
+            except HTTPClientError, ex:
+                assert 'timed out' in ex.args[0]
+            else:
+                assert False, 'HTTPClientError expected'
+            duration2 = time.time() - start
+
+        if sys.version_info >= (2, 6):
+            # check individual timeouts
+            assert 0.1 <= duration1 < 0.2
+            assert 0.2 <= duration2 < 0.3
+        else:
+            # use max timeout in Python 2.5
+            assert 0.2 <= duration1 < 0.3
+            assert 0.2 <= duration2 < 0.3
+
+        mapproxy.client.http._max_set_timeout = old_timeout
+
+        
 OSGEO_CERT = """
 -----BEGIN CERTIFICATE-----
 MIIE2DCCBEGgAwIBAgIEN0rSQzANBgkqhkiG9w0BAQUFADCBwzELMAkGA1UEBhMC
@@ -312,6 +352,23 @@ class TestWMSClient(object):
         img = resp.as_image()
         assert img.mode in ('P', 'RGB')
 
+    def test_similar_srs(self):
+        # request in 3857 and source supports only 900913
+        # 3857 and 900913 are equal but the client requests must use 900913
+        self.req = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo',
+                                    param={'layers':'foo', 'transparent': 'true'})
+        self.wms = WMSClient(self.req, http_client=self.http, supported_srs=[SRS(900913)])
+
+        req = MapQuery((-200000, -200000, 200000, 200000), (512, 512), SRS(3857), 'png')
+        resp = self.wms.get_map(req)
+        eq_(len(self.http.requested), 1)
+        
+        assert_query_eq(self.http.requested[0],
+            TESTSERVER_URL+'/service?map=foo&LAYERS=foo&SERVICE=WMS&FORMAT=image%2Fpng'
+                           '&REQUEST=GetMap&HEIGHT=512&SRS=EPSG%3A900913'
+                           '&VERSION=1.1.1&WIDTH=512&STYLES=&transparent=true'
+                           '&BBOX=-200000,-200000,200000,200000')
+
     def test_transformed_request_transparent(self):
         self.req = WMS111MapRequest(url=TESTSERVER_URL + '/service?map=foo',
                                     param={'layers':'foo', 'transparent': 'true'})
