@@ -15,7 +15,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import division
-from nose.tools import eq_, assert_almost_equal
+from nose.tools import eq_, assert_almost_equal, raises
 from mapproxy.grid import (
     MetaGrid,
     TileGrid,
@@ -24,7 +24,10 @@ from mapproxy.grid import (
     bbox_contains,
     NoTiles,
     tile_grid,
-    resolutions
+    resolutions,
+    ResolutionRange,
+    resolution_range,
+    merge_resolution_range,
 )
 from mapproxy.srs import SRS, TransformationError
 
@@ -600,6 +603,49 @@ class TestTileGrid(object):
         else:
             assert False, 'Expected TransformationError'
 
+class TestTileGridThreshold(object):
+    def test_lower_bound(self):
+        # thresholds near the next lower res value
+        grid = TileGrid(res=[1000, 500, 250, 100, 50], threshold_res=[300, 110])
+        grid.stretch_factor = 1.1
+        eq_(grid.closest_level(1100), 0)
+        # regular transition (w/stretchfactor)
+        eq_(grid.closest_level(950), 0)
+        eq_(grid.closest_level(800), 1)
+        eq_(grid.closest_level(500), 1)
+        # transition at threshold
+        eq_(grid.closest_level(301), 1)
+        eq_(grid.closest_level(300), 2)
+        eq_(grid.closest_level(250), 2)
+        # transition at threshold
+        eq_(grid.closest_level(111), 2)
+        eq_(grid.closest_level(110), 3)
+        eq_(grid.closest_level(100), 3)
+        # regular transition (w/stretchfactor)
+        eq_(grid.closest_level(92), 3)
+        eq_(grid.closest_level(90), 4)
+    def test_upper_bound(self):
+        # thresholds near the next upper res value (within threshold)
+        grid = TileGrid(res=[1000, 500, 250, 100, 50], threshold_res=[495, 240])
+        grid.stretch_factor = 1.1
+        eq_(grid.closest_level(1100), 0)
+        # regular transition (w/stretchfactor)
+        eq_(grid.closest_level(950), 0)
+        eq_(grid.closest_level(800), 1)
+        eq_(grid.closest_level(500), 1)
+        # transition at threshold
+        eq_(grid.closest_level(496), 1)
+        eq_(grid.closest_level(495), 2)
+        eq_(grid.closest_level(250), 2)
+        # transition at threshold (within strechfactor)
+        eq_(grid.closest_level(241), 2)
+        eq_(grid.closest_level(240), 3)
+        eq_(grid.closest_level(100), 3)
+        # regular transition (w/stretchfactor)
+        eq_(grid.closest_level(92), 3)
+        eq_(grid.closest_level(90), 4)
+
+        
 class TestCreateTileList(object):
     def test(self):
         xs = range(-1, 2)
@@ -704,3 +750,104 @@ def assert_almost_equal_bbox(bbox1, bbox2, places=2):
     for coord1, coord2 in zip(bbox1, bbox2):
         assert_almost_equal(coord1, coord2, places)
 
+
+class TestResolutionRange(object):
+    def test_meter(self):
+        res_range = ResolutionRange(1000, 10)
+        assert not res_range.contains([0, 0, 100000, 100000], (10, 10), SRS(900913))
+        assert not res_range.contains([0, 0, 100000, 100000], (99, 99), SRS(900913))
+        # min is exclusive but there is a delta
+        assert     res_range.contains([0, 0, 100000, 100000], (100, 100), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (1000, 1000), SRS(900913))
+        # max is inclusive
+        assert     res_range.contains([0, 0, 100000, 100000], (10000, 10000), SRS(900913))
+        assert not res_range.contains([0, 0, 100000, 100000], (10001, 10001), SRS(900913))
+    def test_deg(self):
+        res_range = ResolutionRange(100000, 1000)
+        assert not res_range.contains([0, 0, 10, 10], (10, 10), SRS(4326))
+        assert not res_range.contains([0, 0, 10, 10], (11, 11), SRS(4326))
+        assert     res_range.contains([0, 0, 10, 10], (12, 12), SRS(4326))
+        assert     res_range.contains([0, 0, 10, 10], (100, 100), SRS(4326))
+        assert     res_range.contains([0, 0, 10, 10], (1000, 1000), SRS(4326))
+        assert     res_range.contains([0, 0, 10, 10], (1100, 1100), SRS(4326))
+        assert not res_range.contains([0, 0, 10, 10], (1200, 1200), SRS(4326))
+    
+    def test_no_min(self):
+        res_range = ResolutionRange(None, 10)
+        assert     res_range.contains([0, 0, 100000, 100000], (1, 1), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (10, 10), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (99, 99), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (100, 100), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (1000, 1000), SRS(900913))
+        # max is inclusive
+        assert     res_range.contains([0, 0, 100000, 100000], (10000, 10000), SRS(900913))
+        assert not res_range.contains([0, 0, 100000, 100000], (10001, 10001), SRS(900913))
+
+    def test_no_max(self):
+        res_range = ResolutionRange(1000, None)
+        assert not res_range.contains([0, 0, 100000, 100000], (10, 10), SRS(900913))
+        assert not res_range.contains([0, 0, 100000, 100000], (99, 99), SRS(900913))
+        # min is exclusive but there is a delta
+        assert     res_range.contains([0, 0, 100000, 100000], (100, 100), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (1000, 1000), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (10000, 10000), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (10001, 10001), SRS(900913))
+        assert     res_range.contains([0, 0, 100000, 100000], (1000000, 100000), SRS(900913))
+
+    def test_none(self):
+        res_range = resolution_range(None, None)
+        assert res_range.contains([0, 0, 100000, 100000], (1, 1), SRS(900913))
+        assert res_range.contains([0, 0, 100000, 100000], (1000000, 100000), SRS(900913))        
+
+    def test_from_scale(self):
+        res_range = resolution_range(max_scale=1e6, min_scale=1e3)
+        assert_almost_equal(res_range.min_res, 280)
+        assert_almost_equal(res_range.max_res, 0.28)
+    
+    @raises(ValueError)
+    def check_invalid_combination(self, min_res, max_res, max_scale, min_scale):
+        resolution_range(min_res, max_res, max_scale, min_scale)
+    
+    def test_invalid_combinations(self):
+        yield self.check_invalid_combination, 10, None, 10, None
+        yield self.check_invalid_combination, 10, 20, 10, None
+        yield self.check_invalid_combination, 10, None, 10, 20
+        yield self.check_invalid_combination, 10, 20, 10, 20
+    
+    @raises(AssertionError)
+    def test_wrong_order_res(self):
+         resolution_range(min_res=10, max_res=100)
+
+    @raises(AssertionError)
+    def test_wrong_order_scale(self):
+         resolution_range(min_scale=100, max_scale=10)
+        
+    
+    def test_merge_resolutions(self):
+        res_range = merge_resolution_range(
+            ResolutionRange(None, 10), ResolutionRange(1000, None))
+        eq_(res_range.min_res, None)
+        eq_(res_range.max_res, None)
+
+        res_range = merge_resolution_range(
+            ResolutionRange(10000, 10), ResolutionRange(1000, None))
+        eq_(res_range.min_res, 10000)
+        eq_(res_range.max_res, None)
+
+        res_range = merge_resolution_range(
+            ResolutionRange(10000, 10), ResolutionRange(1000, 1))
+        eq_(res_range.min_res, 10000)
+        eq_(res_range.max_res, 1)
+        
+        res_range = merge_resolution_range(
+            ResolutionRange(10000, 10), ResolutionRange(None, None))
+        eq_(res_range.min_res, None)
+        eq_(res_range.max_res, None)
+        
+        res_range = merge_resolution_range(
+            None, ResolutionRange(None, None))
+        eq_(res_range, None)
+        
+        res_range = merge_resolution_range(
+            ResolutionRange(10000, 10), None)
+        eq_(res_range, None)
